package sposaro.android.client;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.security.MessageDigest;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Handler;
import android.os.Message;
import android.os.RemoteException;
import android.preference.PreferenceManager;
import android.util.Log;
import android.widget.Toast;

public class FileTransferThread extends Thread {
	
	private int MAX_ATTEMPTS = 3;
	private String file;
	private int port;
	private String addr;
	private int id;
	private String des_dir = "/sdcard/shared/";
	private static String TAG = "sposaro.android.FileTransfer";
	private Handler fileHandler;
	private Context context;

	public FileTransferThread(Context context, Handler fileHandler, String file, int id, String addr, int port){
		this.context = context;
		this.fileHandler = fileHandler;
		this.file = file;
		this.port = port;
		this.addr = addr;
		this.id = id;
	}
	
	boolean download(){
		try {
			// Creating Socket and Communications
			Socket socket = new Socket(addr,port);
			InputStream in = socket.getInputStream();
			PrintWriter out = new PrintWriter(socket.getOutputStream( ), true);
			
			// Requesting file
			out.println(file);
			out.flush();
			   
			// Reading Hash Length
			byte[] sizeBuffer = new byte[4];
			in.read(sizeBuffer);
			
			//Log.i(TAG, "Size of hash is " + byteArrayToInt(sizeBuffer));
			if(byteArrayToInt(sizeBuffer) == -1){
				return false;
			}
			
			// Reading Hash from server
			byte[] fullBuffer = new byte[byteArrayToInt(sizeBuffer)];
			in.read(fullBuffer);
			String serverHash = SimpleSHA1.encode(fullBuffer);
			Log.i(TAG, "Hash From Server is " + serverHash);
			
			
			// Reading file from socket 
			MessageDigest md = MessageDigest.getInstance("SHA1");
			byte[] buffer = new byte[1024];
			FileOutputStream fos = new FileOutputStream(des_dir+file);
			int len;
			
			while((len = in.read(buffer))>0){
				fos.write(buffer,0,len);
				md.update(buffer, 0, len);
			}
			
			// Getting local hash Value
			byte[] mdbytes = md.digest();
			String localHash = SimpleSHA1.encode(mdbytes);
			Log.i(TAG,"Local hash is " + localHash);
			
			// Closing Connections
			fos.close();
			in.close();
			out.close();
			socket.close();
		
			// Comparing hash values
			if(serverHash.equals(localHash))
				return true;
			else
				return false;
			
			
		} catch (Exception e) {
			Log.i(TAG, "FileTransfer Thread " + id + " Unable to Write");
			e.printStackTrace();
		}
		
		
		Log.i(TAG, "Error in Downloading file " + file);
		return false;
	}

	public void run(){
		Log.i(TAG, "FileTransfer Thread " + id + " for file " + file);
		
		Message msg = new Message();
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(context);
		
		if(!prefs.getBoolean("get_bad_data", true)){
			download();
			msg.what = 0;
			fileHandler.sendMessage(msg);
			return;
		}
		
		// Loop to download file and check hash values
		for(int counter = 0; counter < MAX_ATTEMPTS; counter++){
			if(download()){
				Log.i(TAG, "FileTransfer " + file + " complete");
				msg.what = 0;				
				fileHandler.sendMessage(msg);
				return;
			}
			else
				Log.i(TAG, "FileTransfer error for file " + file);
		}
		
		// Cleaning up bad file
		File f = new File(des_dir + file);
		f.delete();

		msg.what = 1;
		fileHandler.sendMessage(msg);
		
		//Toast.makeText(context, "Unable to transfer file " +  file, Toast.LENGTH_LONG).show();
		Log.i(TAG, "Unable to transfer " + file);
	}
	
	public static final int byteArrayToInt(byte [] b) {
        return (b[0] << 24)
                + ((b[1] & 0xFF) << 16)
                + ((b[2] & 0xFF) << 8)
                + (b[3] & 0xFF);
	}
}
