package sposaro.android.client;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

public class ServerFileList extends ListActivity {

	IServerService mService = null;
	private final int MENU_REFRESH = 0;
	private final int MENU_QUIT = 1;
	ProgressDialog progress;
	private boolean show_progress;
	
	Handler myFileHandler = new Handler(){
		public static final int TRANSFER_COMPLETE = 0;
		public static final int TRANSFER_ERROR = 1;
		
		
		
		public void handleMessage(Message msg){
			switch(msg.what){
			case TRANSFER_COMPLETE:
				Toast.makeText(getBaseContext(), "File Transfer Complete", Toast.LENGTH_LONG).show();
				break;
				
			case TRANSFER_ERROR:
				Toast.makeText(getBaseContext(), "File Transfer Error", Toast.LENGTH_LONG).show();
				break;
			}
			
			if(show_progress)
				progress.cancel();
		}
	};
	
	
	private ServiceConnection conn = new ServiceConnection(){
		
		public void onServiceConnected(ComponentName name, IBinder service) {
			Log.i("SERVERFILELIST", "onServiceConnected");
			
			mService = IServerService.Stub.asInterface(service);	
			populateList();
		}

		public void onServiceDisconnected(ComponentName name) {
			Log.i("SERVERFILELIST", "onServiceDisonnected");
			mService = null;
		}
	};
	
	protected void onCreate(Bundle savedInstanceState){
		super.onCreate(savedInstanceState);
		setContentView(R.layout.filelist);
		
		Intent i = new Intent(this, ServerService.class);
		bindService(i, conn,Context.BIND_AUTO_CREATE);
	}
	
	private void populateList(){
		try {
			String []files = mService.getFiles();
			//ArrayAdapter list = new ArrayAdapter<String>(this,android.R.layout.simple_list_item_1,files);
			
			setListAdapter(new ArrayAdapter<String>(this,android.R.layout.simple_list_item_1,files));
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public boolean onCreateOptionsMenu(Menu menu){
		menu.add(0, MENU_REFRESH, 0, "Refresh List").setIcon(android.R.drawable.stat_notify_sync);
		menu.add(0, MENU_QUIT, 0, "Close Connection").setIcon(android.R.drawable.ic_menu_close_clear_cancel);
		return true;
	}
	
	public boolean onOptionsItemSelected(MenuItem item){
		switch(item.getItemId()){
		case MENU_REFRESH:
			populateList();
			return true;
		case MENU_QUIT:
			stopService();
			return true;
		}
		return false;
	}
	
	private void stopService(){
		conn = null;
		stopService(new Intent(this,ServerService.class));
		Toast.makeText(this, "Connection Closed", Toast.LENGTH_LONG).show();
		
		finish();
	}
	
	protected void onListItemClick(ListView lv, View v, int position, long id){
		
		TextView text = (TextView)v;
		
		String []client = null;
		String fileName = "" + text.getText();
		
		try {
			client = mService.query(fileName);
			
			if(client != null){
				if(show_progress = PreferenceManager.getDefaultSharedPreferences(getBaseContext()).getBoolean("show_progress", true))
					progress = ProgressDialog.show(this, "Download", "Transfering file " + fileName + " From Client " + client[0]);
				new FileTransferThread(this, myFileHandler, "" + fileName,Integer.parseInt(client[0]),client[1],Integer.parseInt(client[2])).start();
			}
			else
				return;
			
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}
