package sposaro.android.client;

import java.io.File;
import android.app.Service;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.IBinder;
import android.os.RemoteException;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

public class ServerService extends Service {

	private Network n;
	private int myPORT;
	public static int hostPort;
	public static String TAG = "sposaro.android.client.ServerService";
	public static String scr_dir = "/sdcard/shared";
	private PhoneServer server;
	protected static boolean running;
	
	
	private final IServerService.Stub mBinder = new IServerService.Stub() {
		
		public String[] getFiles() throws RemoteException {
			n.send(":list:");
			
        	String s;	
        	while((s = n.recv()) == null)
        		Log.i(TAG, "waiting for input");
        
			return s.split(",");	
		}
		
		public String[] query(String fileName){
			if(findFile(fileName)){
				Toast.makeText(getBaseContext(), "You already have file \"" + fileName + "\"", Toast.LENGTH_SHORT).show();
				return null;
			}
			else
				n.send(":query:" + fileName);
			
			String s;
			
			while((s = n.recv()) == null)
        		Log.i(TAG, "waiting for input");
			
			return s.split(",");
		}

		public void insert(String fileName) throws RemoteException {
			n.send(":insert:" + fileName);
		}

		public boolean searchFile(String fileName) throws RemoteException {
			return findFile(scr_dir+fileName);
		}
	};
	
	
	 public void close(View v){
		 
		 n.send("q");
	    	n.stop();
	    }
	 
	    public boolean findFile(String name){
	    	
	    	File dir = new File(scr_dir); 
			String[] children = dir.list(); 
			
			if (children != null) { 
				for (int i=0; i<children.length; i++) 
				{ // Get filename of file or directory 
					if(name.equals(children[i]))
						return true;
				} 
			} 
	    	
	    	return false;
	    }
	
	public int onStartCommand(Intent intent, int flags, int startId){
		running = true;
		Log.i(TAG, "Starting Server Service");
		SharedPreferences pref = PreferenceManager.getDefaultSharedPreferences(this);
		
		
		n = new Network(pref.getString("server_name", "Nothing has Been Entered"), Integer.parseInt(pref.getString("server_port", "Nothing Has Been Entered")));
		
		myPORT = Integer.parseInt(pref.getString("my_port", "Nothing Has Been Entered"));		
		
		Log.i(TAG, "myport is <" + myPORT + ">");
		
		n.send(":port:" + myPORT);
		server = new PhoneServer(myPORT,this);
		server.start();
		
		writeFiles();
		
		return 0;
	}
	

	public void onDestroy(){
		n.send(":quit:");
		n.stop();
		server.close();
		server.listening = false;	
		running = false;
	}
	
	 public void writeFiles(){
	    	
	    	Log.i(TAG,"attempting to send files");
	    	 
	    	if(n.connected){
	    		File dir = new File(scr_dir); 
	    		String[] children = dir.list(); 
	    		String list = ":insert:";
	    		
	    		if (children != null) { 
	    			for (int i=0; i<children.length; i++) 
	    			{ // Get filename of file or directory 
	    				Log.i(TAG,"Sending filename " + children[i]);
	    				list += "," + children[i];
	    			} 
	    			list = list.replaceFirst(",", "");
	    			n.send(list);
	    		}
	    	} 
	    	 
	    }   
	    

	@Override
	public IBinder onBind(Intent intent) {
		running = true;
		return mBinder;
	}

}
