/*
 * Copyright (c) 1995 - 2008 Sun Microsystems, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Sun Microsystems nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */ 

import java.net.*;
import java.io.*;
import java.util.*;

public class KKMultiServerThread extends Thread {
    private Socket socket = null;
    private Data data;
    private Client client;
    
    public KKMultiServerThread(Socket socket, int id, Data data) {
	super("KKMultiServerThread");
	this.socket = socket;
	this.data = data;

	this.client = new Client();

	client.id = id;
	client.addr = socket.getInetAddress();
	
	System.out.println("Client " + client.id + " connected with address " + socket.getInetAddress());
    }

    public void run() {

	try {
	    PrintWriter out = new PrintWriter(socket.getOutputStream(), true);
	    BufferedReader in = new BufferedReader(
				    new InputStreamReader(
				    socket.getInputStream()));

	    String inputLine, outputLine;

	    while ((inputLine = in.readLine()) != null) {
		outputLine = inputLine;
		System.out.println("Received <" + outputLine + "> from client " + client.id);


		if (outputLine.contains(":quit:"))
		    break;
		else
		if(outputLine.contains(":query:")){
		    outputLine = outputLine.replaceAll(":query:","");
		    System.out.println("\tProcessing Query For File <" + outputLine + "> From Client " + client.id);
		    
		    List<Client> seeders = data.get(outputLine);

		    if(seeders == null){
			System.out.println("\tUnknown File <" + outputLine + ">");
			out.write(-1);			
		    }
		    else{
			Client src = seeders.get(0);
			String result = src.id + "," + src.addr.getHostAddress() + "," + src.port;
			System.out.println("\tQuery Result Client " + result);
			out.println(result);
		    }
		    out.flush();
		}
		else
		if(outputLine.contains(":list:")){
		    System.out.println("\tProcessing File List Request From Client " + client.id);


		    String list = data.getFiles();
		    list = list.replaceFirst(",","");
		    System.out.println("\tList Request Result <"+ list +">");		    
		    out.flush();
		    out.println(list);
		    out.flush();
		}
		else
		    if(outputLine.contains(":port:")){
			outputLine = outputLine.replace(":port:", "");

			client.port = Integer.parseInt(outputLine);

			System.out.println("\tChanging listport to " + client.port);
		    }
		else
		if(outputLine.contains(":insert:")){
		    outputLine = outputLine.replace(":insert:", "");
		    String [] files = outputLine.split(",");

		    for(String s : files){
			System.out.println("\tclient " + client.id + " inserting file <" + s + ">");			
			data.put(s,client);
		    }
		    

		}		   
	    }
	    out.close();
	    in.close();
	    socket.close();

	    System.out.println("Closing Client " + client.id);
	    data.remove(client);
	    client = null;


	} catch (IOException e) {
	    e.printStackTrace();
	}
    }
}
